/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.fluent;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.sourceprojects.lycia.LyciaConfigurator;
import org.sourceprojects.lycia.LyciaParser;
import org.sourceprojects.lycia.TagDefinitionParser;
import org.sourceprojects.lycia.exceptions.LyciaInitializationException;
import org.sourceprojects.lycia.internal.privileged.SetAccessible;

class TagDefinitionParserEventBuilder<B> implements EventBuilder<B> {
	private final TagDefinitionParser<B> tagDefinitionParser;

	TagDefinitionParserEventBuilder(
			final Class<? extends TagDefinitionParser<B>> tagDefinitionParserClass) {

		try {
			Constructor<? extends TagDefinitionParser<B>> constructor;
			constructor = tagDefinitionParserClass.getConstructor();

			if (!constructor.isAccessible())
				SetAccessible.setAccessible(constructor);

			this.tagDefinitionParser = constructor.newInstance();

		} catch (final InstantiationException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		} catch (final IllegalAccessException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		} catch (SecurityException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		} catch (NoSuchMethodException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		} catch (IllegalArgumentException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		} catch (InvocationTargetException e) {
			throw new LyciaInitializationException(
					"POJO parser could not be initialized", e);
		}
	}

	TagDefinitionParserEventBuilder(
			final TagDefinitionParser<B> tagDefinitionParser) {
		this.tagDefinitionParser = tagDefinitionParser;
	}

	public TagDefinitionParser<B> build(
			final LyciaConfigurator<B> configurator,
			final LyciaParser<B> lyciaParser) {

		return tagDefinitionParser;
	}
}
