/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal;

import java.io.IOException;

import org.sourceprojects.lycia.ResourceResolver;
import org.sourceprojects.lycia.internal.dom.ResourceResolverResult;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.ext.EntityResolver2;

/**
 * A basic implementation of the {@link EntityResolver2} interface.
 * 
 * @author noctarius
 * @since 0.0.1
 */
public class EntityResolverImpl implements EntityResolver {
	private final ResourceResolver resourceResolver;

	public EntityResolverImpl(final ResourceResolver resourceResolver) {
		this.resourceResolver = resourceResolver;
	}

	@Override
	public InputSource resolveEntity(final String publicId,
			final String systemId) throws SAXException, IOException {

		final ResourceResolverResult input = resourceResolver.resolveResource(
				null, null, publicId, systemId, null);

		if (input != null && input.getByteStream() != null) {
			return new InputSource(input.getByteStream());
		}

		return null;
	}

}
