/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal.dataconverter;

import org.sourceprojects.lycia.DataConverter;

public class BaseDataConverter implements DataConverter {
	public static final DataConverter BASE_DATA_CONVERTER = new BaseDataConverter();

	private BaseDataConverter() {
	}

	@Override
	public Object convert(final Object value, final Class<?> type)
			throws Exception {
		if (type.equals(String.class)) {
			return value;
		}

		if (type.equals(byte[].class)) {
			if (value.getClass().equals(byte[].class)) {
				return value;
			} else {
				return ((String) value).getBytes();
			}
		}

		if (type.equals(String[].class)) {
			return ((String) value).split(",");
		}

		if (type.equals(short.class) || type.equals(Short.class)) {
			return Short.parseShort((String) value);
		}

		if (type.equals(int.class) || type.equals(Integer.class)) {
			return Integer.parseInt((String) value);
		}

		if (type.equals(long.class) || type.equals(Long.class)) {
			return Long.parseLong((String) value);
		}

		if (type.equals(float.class) || type.equals(Float.class)) {
			return Float.parseFloat((String) value);
		}

		if (type.equals(double.class) || type.equals(Double.class)) {
			return Double.parseDouble((String) value);
		}

		if (type.equals(char[].class)) {
			return ((String) value).toCharArray();
		}

		return null;
	}

}
