/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal.validation;

import java.util.regex.Matcher;

import org.sourceprojects.lycia.ConstraintValidator;
import org.sourceprojects.lycia.annotations.Pattern;
import org.sourceprojects.lycia.annotations.Pattern.Flag;

/**
 * 
 * @author noctarius
 * @since 1.9.1
 */
public class PatternConstraintValidator implements ConstraintValidator<Pattern> {

	private java.util.regex.Pattern pattern;

	@Override
	public void initialize(final Pattern annotation) {
		int flags = 0;

		for (final Flag flag : annotation.flags()) {
			flags |= getFlagValue(flag);
		}

		pattern = java.util.regex.Pattern.compile(annotation.regexp(), flags);
	}

	@Override
	public boolean isValid(final Object object) {
		if (object == null) {
			return true;
		}

		final String value = object.toString();
		final Matcher matcher = pattern.matcher(value);
		return matcher.matches();
	}

	private int getFlagValue(final Flag flag) {
		switch (flag) {
		case CANON_EQ:
			return java.util.regex.Pattern.CANON_EQ;
		case CASE_INSENSITIVE:
			return java.util.regex.Pattern.CASE_INSENSITIVE;
		case COMMENTS:
			return java.util.regex.Pattern.COMMENTS;
		case DOTALL:
			return java.util.regex.Pattern.DOTALL;
		case MULTILINE:
			return java.util.regex.Pattern.MULTILINE;
		case UNICODE_CASE:
			return java.util.regex.Pattern.UNICODE_CASE;
		case UNIX_LINES:
			return java.util.regex.Pattern.UNIX_LINES;
		default:
			return 0;
		}
	}

}
